import requests
import csv
import re
from prettytable import PrettyTable
from tqdm import tqdm

def fetch_models(make, timeout=10):
    url = f"https://www.carqueryapi.com/api/0.3/?cmd=getModels&make={make}"
    try:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
        }
        response = requests.get(url, headers=headers, timeout=timeout)
        response.raise_for_status()
        data = response.json()
        if 'Models' in data:
            return [model['model_name'] for model in data['Models']]
        else:
            print("No models found for the provided manufacturer.")
            return []
    except requests.exceptions.RequestException as e:
        print(f"Error fetching data: {e}")
        return []

def fetch_car_data(make="", model="", variant="", from_year=None, to_year=None, sold_in_us=None, timeout=50):
    car_data = []
    if model:
        car_data.extend(fetch_variants(make, model, variant, from_year, to_year, sold_in_us, timeout))
    else:
        models = fetch_models(make, timeout)
        for model_name in tqdm(models, desc="Fetching variants"):
            car_data.extend(fetch_variants(make, model_name, variant, from_year, to_year, sold_in_us, timeout))
    return car_data

def fetch_variants(make, model, variant="", from_year=None, to_year=None, sold_in_us=None, timeout=50):
    url = f"https://www.carqueryapi.com/api/0.3/?cmd=getTrims&make={make}&model={model}"
    try:
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
        }
        response = requests.get(url, headers=headers, timeout=timeout)
        response.raise_for_status()
        data = response.json()
        variants = []
        if 'Trims' in data:
            for variant_info in tqdm(data['Trims'], desc="Processing variants"):
                model_year = int(variant_info['model_year'])
                sold_in_us_bool = variant_info.get('sold_in_us', '').lower() == 'yes'
                if (variant == "" or variant.lower() == variant_info['model_trim'].lower()) and \
                   ((from_year is None or model_year >= from_year) and (to_year is None or model_year <= to_year)) and \
                   (sold_in_us is None or sold_in_us_bool == sold_in_us):
                    variants.append({
                        'vehicle_produced_since': variant_info['model_year'],
                        'vehicle_produced_until': "",
                        'vehicle_brand': make,
                        'vehicle_model': model,
                        'vehicle_variant': variant_info['model_trim']
                    })
    except requests.exceptions.RequestException as e:
        print(f"Error fetching variants: {e}")
    return variants

def write_to_csv(filename, data):
    with open(filename, 'w', newline='') as csvfile:
        fieldnames = ['vehicle_produced_since', 'vehicle_produced_until', 'vehicle_brand', 'vehicle_model', 'vehicle_variant']
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        for item in data:
            writer.writerow(item)

def sanitize_filename(filename):
    return re.sub(r'[^a-zA-Z0-9]+', '-', filename.strip())

def print_table(data):
    table = PrettyTable()
    table.field_names = ["Model"]
    for item in data:
        table.add_row([item])
    print(table)
